class Global():
    @classmethod
    def get_size(cls, bytes_int, suffix="B"):
        """Scale bytes to its proper format

        e.g:
            1253656 => '1.20MB'
            1253656678 => '1.17GB'
        """
        factor = 1024
        for unit in ['', 'K', 'M', 'G', 'T', 'P']:
            if bytes_int < factor:
                return f'{bytes_int:.2f}{unit}{suffix}'
            bytes_int /= factor



import psutil


class Cpu():
    @classmethod
    def serve(cls):
        """Serve CPU info
        """
        # Title
        cpu_title = '='*15 + ' CPU Information ' + '='*15

        # Cores
        physical_cores = 'Physical cores:' + \
            str(psutil.cpu_count(logical=False))
        total_cores = 'Total cores:' + str(psutil.cpu_count(logical=True))

        # CPU Frequencies
        cpufreq = psutil.cpu_freq()
        max_freq = f'Max Frequency: {cpufreq.max:.2f}Mhz'
        min_freq = f'Min Frequency: {cpufreq.min:.2f}Mhz'
        current_freq = f'Current Frequency: {cpufreq.current:.2f}Mhz'

        # CPU Usage
        usage_message = 'CPU Usage Per Core:'
        all_core_percentage = ''
        for i, percentage in enumerate(
            psutil.cpu_percent(
                percpu=True,
                interval=1
            )
        ):
            core_percentage = f'Core {i}: {percentage}%\n'
            # Combine each core into a variable
            all_core_percentage += core_percentage
        total_usage = f'Total CPU Usage: {psutil.cpu_percent()}%'

        final_message = (
            '\n' + cpu_title +
            '\n' + physical_cores +
            '\n' + total_cores +
            '\n' + max_freq +
            '\n' + min_freq +
            '\n' + current_freq +
            '\n' + usage_message +
            '\n' + all_core_percentage + total_usage
        )
        return final_message



import os
import slack
from dotenv import load_dotenv


SLACK_BOT_TOKEN = os.getenv('SLACK_BOT_TOKEN')
SLACK_CHANNEL = os.getenv('SLACK_CHANNEL')


class Slack():
    @classmethod
    def message(cls, message):
        """Send the output of Service to Slack
        """
        load_dotenv()
        slack_client = slack.WebClient(SLACK_BOT_TOKEN)
        try:
            slack_client.chat_postMessage(
                channel=SLACK_CHANNEL,
                text=message
            )
            success = 'Slack message sent!'
            return success
        except slack.errors.SlackApiError:
            raise Exception('Could not send the Service Slack message.')



import psutil
from service.stats.globals import Global


class Memory():
    @classmethod
    def serve(cls):
        """Serve memory (RAM) info
        """
        # Title
        memory_title = '='*15 + ' Memory Information ' + '='*15
        swap_title = '='*8 + ' SWAP ' + '='*8

        # Memory Information
        svmem = psutil.virtual_memory()
        memory_total = f'Total: {Global.get_size(svmem.total)}'
        memory_available = f'Available: {Global.get_size(svmem.available)}'
        memory_used = f'Used: {Global.get_size(svmem.used)}'
        memory_percentage = f'Percentage: {svmem.percent}%'

        # Get the swap memory details (if it exists)
        swap = psutil.swap_memory()
        swap_total = f'Total: {Global.get_size(swap.total)}'
        swap_free = f'Free: {Global.get_size(swap.free)}'
        swap_used = f'Used: {Global.get_size(swap.used)}'
        swap_percentage = f'Percentage: {swap.percent}%'

        final_message = (
            '\n' + memory_title +
            '\n' + memory_total +
            '\n' + memory_available +
            '\n' + memory_used +
            '\n' + memory_percentage +
            '\n' + swap_title +
            '\n' + swap_total +
            '\n' + swap_free +
            '\n' + swap_used +
            '\n' + swap_percentage
        )
        return final_message



import mock
import pytest
from service.slack import Slack


def test_slack_error():
    with pytest.raises(Exception):
        Slack.message('dummy message')


@mock.patch('service.slack.SLACK_BOT_TOKEN', '123')
@mock.patch('service.slack.SLACK_CHANNEL', 'mock-channel')
@mock.patch('slack.WebClient.chat_postMessage')
def test_slack_success(mock_slack):
    message = 'dummy message'
    Slack.message(message)
    mock_slack.assert_called_once_with(channel='mock-channel', text=message)
